@@ -33,7 +33,7 @@ class JupyterConsole(BaseExtension):
     
     preferences_ui = 'extensions.JupyterConsole.preferences'
 
-    @BaseExtension.as_thread(wait=2000)
+    @BaseExtension.as_thread(wait=500)
     def event_startup(self):
 
         from jupyter_tabwidget import ConsoleTabWidget
@@ -87,13 +87,21 @@ class JupyterConsole(BaseExtension):
 
         self._jupyter_console.add(kernel=kernel)
 
-    def event_jupyter_run_file(self, path):
+    def event_jupyter_run_file(self, path, debug=False):
 
         self._set_visible(True)
         if not os.path.isfile(path):
             return
         self._jupyter_console.current.change_dir(os.path.dirname(path))
-        self._jupyter_console.current.run_file(path)
+        if debug:
+            self._jupyter_console.current.run_debug(
+                path,
+                breakpoints=self.extension_manager.provide(
+                    'pyqode_breakpoints'
+                )
+            )
+        else:
+            self._jupyter_console.current.run_file(path)
 
     def event_jupyter_change_dir(self, path):
 
@@ -137,14 +145,32 @@ class JupyterConsole(BaseExtension):
             return self._jupyter_console.current.name
         except AttributeError:
             return None
+        
+    def provide_workspace_kernel(self):
+        
+        try:
+            return self._jupyter_console.current._kernel
+        except AttributeError:
+            return None
 
     def provide_jupyter_workspace_globals(self):
 
         return self.get_workspace_globals()
 
+    def provide_jupyter_list_workspace_globals(self):
+
+        return self.list_workspace_globals()
+
     def provide_jupyter_workspace_variable(self, name):
 
         return self._jupyter_console.current.get_workspace_variable(name)
+        
+    def provide_jupyter_kernel_running(self):
+        
+        try:
+            return self._jupyter_console.current.running()
+        except AttributeError:
+            return False
 
     def provide_jupyter_check_syntax(self, code):
 
@@ -157,6 +183,14 @@ class JupyterConsole(BaseExtension):
         except AttributeError:
             return {u'no reply': None}
 
+    def list_workspace_globals(self):
+
+        try:
+            return self._jupyter_console.current.list_workspace_globals()
+        except Exception as e:
+            print(e)
+            return []
+
     def event_close(self):
 
         if not hasattr(self, '_jupyter_console'):
